// Copyright 2002 Adam Megacz, see the COPYING file for licensing [GPL]
package org.ibex.graphics;
import java.util.*;
import org.ibex.util.*;

public class Color {
    
    public static int stringToColor(String s) {
        // FIXME support three-char strings by doubling digits
        if (s == null) return 0x00000000;
        else if (standard.get(s) != null) return 0xFF000000 | ((Integer)standard.get(s)).intValue();
        else if (s.length() == 7 && s.charAt(0) == '#') try {
            // FEATURE  alpha
            return 0xFF000000 |
                (Integer.parseInt(s.substring(1, 3), 16) << 16) |
                (Integer.parseInt(s.substring(3, 5), 16) << 8) |
                Integer.parseInt(s.substring(5, 7), 16);
        } catch (NumberFormatException e) {
            Log.info(Color.class, "invalid color " + s);
            return 0;
        }
        else return 0; // FEATURE: error?
    }

    public static String colorToString(int argb) {
        if ((argb & 0xFF000000) == 0) return null;
        String red = Integer.toHexString((argb & 0x00FF0000) >> 16);
        String green = Integer.toHexString((argb & 0x0000FF00) >> 8);
        String blue = Integer.toHexString(argb & 0x000000FF);
        if (red.length() < 2) red = "0" + red;
        if (blue.length() < 2) blue = "0" + blue;
        if (green.length() < 2) green = "0" + green;
        return "#" + red + green + blue;
    }

    /** Copied verbatim from the SVG specification */
    private static Hashtable standard = new Hashtable(400);
    static {
	standard.put("aliceblue", new Integer((240 << 16) | (248 << 8) | 255));
	standard.put("antiquewhite", new Integer((250 << 16) | (235 << 8) | 215));
	standard.put("aqua", new Integer((0 << 16) | (255 << 8) | 255));
	standard.put("aquamarine", new Integer((127 << 16) | (255 << 8) | 212));
	standard.put("azure", new Integer((240 << 16) | (255 << 8) | 255));
	standard.put("beige", new Integer((245 << 16) | (245 << 8) | 220));
	standard.put("bisque", new Integer((255 << 16) | (228 << 8) | 196));
	standard.put("black", new Integer((0 << 16) | (0 << 8) | 0));
	standard.put("blanchedalmond", new Integer((255 << 16) | (235 << 8) | 205));
	standard.put("blue", new Integer((0 << 16) | (0 << 8) | 255));
	standard.put("blueviolet", new Integer((138 << 16) | (43 << 8) | 226));
	standard.put("brown", new Integer((165 << 16) | (42 << 8) | 42));
	standard.put("burlywood", new Integer((222 << 16) | (184 << 8) | 135));
	standard.put("cadetblue", new Integer((95 << 16) | (158 << 8) | 160));
	standard.put("chartreuse", new Integer((127 << 16) | (255 << 8) | 0));
	standard.put("chocolate", new Integer((210 << 16) | (105 << 8) | 30));
	standard.put("coral", new Integer((255 << 16) | (127 << 8) | 80));
	standard.put("cornflowerblue", new Integer((100 << 16) | (149 << 8) | 237));
	standard.put("cornsilk", new Integer((255 << 16) | (248 << 8) | 220));
	standard.put("crimson", new Integer((220 << 16) | (20 << 8) | 60));
	standard.put("cyan", new Integer((0 << 16) | (255 << 8) | 255));
	standard.put("darkblue", new Integer((0 << 16) | (0 << 8) | 139));
	standard.put("darkcyan", new Integer((0 << 16) | (139 << 8) | 139));
	standard.put("darkgoldenrod", new Integer((184 << 16) | (134 << 8) | 11));
	standard.put("darkgray", new Integer((169 << 16) | (169 << 8) | 169));
	standard.put("darkgreen", new Integer((0 << 16) | (100 << 8) | 0));
	standard.put("darkgrey", new Integer((169 << 16) | (169 << 8) | 169));
	standard.put("darkkhaki", new Integer((189 << 16) | (183 << 8) | 107));
	standard.put("darkmagenta", new Integer((139 << 16) | (0 << 8) | 139));
	standard.put("darkolivegreen", new Integer((85 << 16) | (107 << 8) | 47));
	standard.put("darkorange", new Integer((255 << 16) | (140 << 8) | 0));
	standard.put("darkorchid", new Integer((153 << 16) | (50 << 8) | 204));
	standard.put("darkred", new Integer((139 << 16) | (0 << 8) | 0));
	standard.put("darksalmon", new Integer((233 << 16) | (150 << 8) | 122));
	standard.put("darkseagreen", new Integer((143 << 16) | (188 << 8) | 143));
	standard.put("darkslateblue", new Integer((72 << 16) | (61 << 8) | 139));
	standard.put("darkslategray", new Integer((47 << 16) | (79 << 8) | 79));
	standard.put("darkslategrey", new Integer((47 << 16) | (79 << 8) | 79));
	standard.put("darkturquoise", new Integer((0 << 16) | (206 << 8) | 209));
	standard.put("darkviolet", new Integer((148 << 16) | (0 << 8) | 211));
	standard.put("deeppink", new Integer((255 << 16) | (20 << 8) | 147));
	standard.put("deepskyblue", new Integer((0 << 16) | (191 << 8) | 255));
	standard.put("dimgray", new Integer((105 << 16) | (105 << 8) | 105));
	standard.put("dimgrey", new Integer((105 << 16) | (105 << 8) | 105));
	standard.put("dodgerblue", new Integer((30 << 16) | (144 << 8) | 255));
	standard.put("firebrick", new Integer((178 << 16) | (34 << 8) | 34));
	standard.put("floralwhite", new Integer((255 << 16) | (250 << 8) | 240));
	standard.put("forestgreen", new Integer((34 << 16) | (139 << 8) | 34));
	standard.put("fuchsia", new Integer((255 << 16) | (0 << 8) | 255));
	standard.put("gainsboro", new Integer((220 << 16) | (220 << 8) | 220));
	standard.put("ghostwhite", new Integer((248 << 16) | (248 << 8) | 255));
	standard.put("gold", new Integer((255 << 16) | (215 << 8) | 0));
	standard.put("goldenrod", new Integer((218 << 16) | (165 << 8) | 32));
	standard.put("gray", new Integer((128 << 16) | (128 << 8) | 128));
	standard.put("grey", new Integer((128 << 16) | (128 << 8) | 128));
	standard.put("green", new Integer((0 << 16) | (128 << 8) | 0));
	standard.put("greenyellow", new Integer((173 << 16) | (255 << 8) | 47));
	standard.put("honeydew", new Integer((240 << 16) | (255 << 8) | 240));
	standard.put("hotpink", new Integer((255 << 16) | (105 << 8) | 180));
	standard.put("indianred", new Integer((205 << 16) | (92 << 8) | 92));
	standard.put("indigo", new Integer((75 << 16) | (0 << 8) | 130));
	standard.put("ivory", new Integer((255 << 16) | (255 << 8) | 240));
	standard.put("khaki", new Integer((240 << 16) | (230 << 8) | 140));
	standard.put("lavender", new Integer((230 << 16) | (230 << 8) | 250));
	standard.put("lavenderblush", new Integer((255 << 16) | (240 << 8) | 245));
	standard.put("lawngreen", new Integer((124 << 16) | (252 << 8) | 0));
	standard.put("lemonchiffon", new Integer((255 << 16) | (250 << 8) | 205));
	standard.put("lightblue", new Integer((173 << 16) | (216 << 8) | 230));
	standard.put("lightcoral", new Integer((240 << 16) | (128 << 8) | 128));
	standard.put("lightcyan", new Integer((224 << 16) | (255 << 8) | 255));
	standard.put("lightgoldenrodyellow", new Integer((250 << 16) | (250 << 8) | 210));
	standard.put("lightgray", new Integer((211 << 16) | (211 << 8) | 211));
	standard.put("lightgreen", new Integer((144 << 16) | (238 << 8) | 144));
	standard.put("lightgrey", new Integer((211 << 16) | (211 << 8) | 211));
	standard.put("lightpink", new Integer((255 << 16) | (182 << 8) | 193));
	standard.put("lightsalmon", new Integer((255 << 16) | (160 << 8) | 122));
	standard.put("lightseagreen", new Integer((32 << 16) | (178 << 8) | 170));
	standard.put("lightskyblue", new Integer((135 << 16) | (206 << 8) | 250));
	standard.put("lightslategray", new Integer((119 << 16) | (136 << 8) | 153));
	standard.put("lightslategrey", new Integer((119 << 16) | (136 << 8) | 153));
	standard.put("lightsteelblue", new Integer((176 << 16) | (196 << 8) | 222));
	standard.put("lightyellow", new Integer((255 << 16) | (255 << 8) | 224));
	standard.put("lime", new Integer((0 << 16) | (255 << 8) | 0));
	standard.put("limegreen", new Integer((50 << 16) | (205 << 8) | 50));
	standard.put("linen", new Integer((250 << 16) | (240 << 8) | 230));
	standard.put("magenta", new Integer((255 << 16) | (0 << 8) | 255));
	standard.put("maroon", new Integer((128 << 16) | (0 << 8) | 0));
	standard.put("mediumaquamarine", new Integer((102 << 16) | (205 << 8) | 170));
	standard.put("mediumblue", new Integer((0 << 16) | (0 << 8) | 205));
	standard.put("mediumorchid", new Integer((186 << 16) | (85 << 8) | 211));
	standard.put("mediumpurple", new Integer((147 << 16) | (112 << 8) | 219));
	standard.put("mediumseagreen", new Integer((60 << 16) | (179 << 8) | 113));
	standard.put("mediumslateblue", new Integer((123 << 16) | (104 << 8) | 238));
	standard.put("mediumspringgreen", new Integer((0 << 16) | (250 << 8) | 154));
	standard.put("mediumturquoise", new Integer((72 << 16) | (209 << 8) | 204));
	standard.put("mediumvioletred", new Integer((199 << 16) | (21 << 8) | 133));
	standard.put("midnightblue", new Integer((25 << 16) | (25 << 8) | 112));
	standard.put("mintcream", new Integer((245 << 16) | (255 << 8) | 250));
	standard.put("mistyrose", new Integer((255 << 16) | (228 << 8) | 225));
	standard.put("moccasin", new Integer((255 << 16) | (228 << 8) | 181));
	standard.put("navajowhite", new Integer((255 << 16) | (222 << 8) | 173));
	standard.put("navy", new Integer((0 << 16) | (0 << 8) | 128));
	standard.put("oldlace", new Integer((253 << 16) | (245 << 8) | 230));
	standard.put("olive", new Integer((128 << 16) | (128 << 8) | 0));
	standard.put("olivedrab", new Integer((107 << 16) | (142 << 8) | 35));
	standard.put("orange", new Integer((255 << 16) | (165 << 8) | 0));
	standard.put("orangered", new Integer((255 << 16) | (69 << 8) | 0));
	standard.put("orchid", new Integer((218 << 16) | (112 << 8) | 214));
	standard.put("palegoldenrod", new Integer((238 << 16) | (232 << 8) | 170));
	standard.put("palegreen", new Integer((152 << 16) | (251 << 8) | 152));
	standard.put("paleturquoise", new Integer((175 << 16) | (238 << 8) | 238));
	standard.put("palevioletred", new Integer((219 << 16) | (112 << 8) | 147));
	standard.put("papayawhip", new Integer((255 << 16) | (239 << 8) | 213));
	standard.put("peachpuff", new Integer((255 << 16) | (218 << 8) | 185));
	standard.put("peru", new Integer((205 << 16) | (133 << 8) | 63));
	standard.put("pink", new Integer((255 << 16) | (192 << 8) | 203));
	standard.put("plum", new Integer((221 << 16) | (160 << 8) | 221));
	standard.put("powderblue", new Integer((176 << 16) | (224 << 8) | 230));
	standard.put("purple", new Integer((128 << 16) | (0 << 8) | 128));
	standard.put("red", new Integer((255 << 16) | (0 << 8) | 0));
	standard.put("rosybrown", new Integer((188 << 16) | (143 << 8) | 143));
	standard.put("royalblue", new Integer((65 << 16) | (105 << 8) | 225));
	standard.put("saddlebrown", new Integer((139 << 16) | (69 << 8) | 19));
	standard.put("salmon", new Integer((250 << 16) | (128 << 8) | 114));
	standard.put("sandybrown", new Integer((244 << 16) | (164 << 8) | 96));
	standard.put("seagreen", new Integer((46 << 16) | (139 << 8) | 87));
	standard.put("seashell", new Integer((255 << 16) | (245 << 8) | 238));
	standard.put("sienna", new Integer((160 << 16) | (82 << 8) | 45));
	standard.put("silver", new Integer((192 << 16) | (192 << 8) | 192));
	standard.put("skyblue", new Integer((135 << 16) | (206 << 8) | 235));
	standard.put("slateblue", new Integer((106 << 16) | (90 << 8) | 205));
	standard.put("slategray", new Integer((112 << 16) | (128 << 8) | 144));
	standard.put("slategrey", new Integer((112 << 16) | (128 << 8) | 144));
	standard.put("snow", new Integer((255 << 16) | (250 << 8) | 250));
	standard.put("springgreen", new Integer((0 << 16) | (255 << 8) | 127));
	standard.put("steelblue", new Integer((70 << 16) | (130 << 8) | 180));
	standard.put("tan", new Integer((210 << 16) | (180 << 8) | 140));
	standard.put("teal", new Integer((0 << 16) | (128 << 8) | 128));
	standard.put("thistle", new Integer((216 << 16) | (191 << 8) | 216));
	standard.put("tomato", new Integer((255 << 16) | (99 << 8) | 71));
	standard.put("turquoise", new Integer((64 << 16) | (224 << 8) | 208));
	standard.put("violet", new Integer((238 << 16) | (130 << 8) | 238));
	standard.put("wheat", new Integer((245 << 16) | (222 << 8) | 179));
	standard.put("white", new Integer((255 << 16) | (255 << 8) | 255));
	standard.put("whitesmoke", new Integer((245 << 16) | (245 << 8) | 245));
	standard.put("yellow", new Integer((255 << 16) | (255 << 8) | 0));
	standard.put("yellowgreen", new Integer((154 << 16) | (205 << 8) | 50));
    }
}
